#* vim:ts=4:
#*---------------------------------------------------------------------------*
#* Project
#*        slurpmedia - Slurp multimedia files from news articles
#*        Copyright (C) 1999  klmcw@earthlink.net <Kevin L. McWhirter>
#*
#*        This program is free software; you can redistribute it and/or modify
#*        it under the terms of the GNU General Public License as published by
#*        the Free Software Foundation; either version 2 of the License, or
#*        (at your option) any later version.
#*
#*        This program is distributed in the hope that it will be useful,
#*        but WITHOUT ANY WARRANTY; without even the implied warranty of
#*        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*        GNU General Public License for more details.
#*
#*        You should have received a copy of the GNU General Public License
#*        along with this program; if not, write to the Free Software
#*        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#*
#* Module
#*        $Id$
#*
#* Description
#*
#*---------------------------------------------------------------------------*
#* $Log$
#*---------------------------------------------------------------------------*
package smApp;
use strict;
use vars qw/$AUTOLOAD/;

use Carp;
use File::Path;
use File::Spec;

require News::Newsrc;

require smfmMain;
require smNewsFacade;
require smOpts;

$smApp::VERSION = (qw$Revision: 0.3$)[1];
$smApp::VDATE = (qw$Date: 1999/08/22 00:00:00$)[1];

#*---------------------------------------------------------------------------*
sub AUTOLOAD {
	my $self = shift;
	my $type = ref($self) or croak "$self is not an object";
	my $name = $AUTOLOAD;

	$name =~ s/.*://;   # strip fully-qualified portion

	return $self->{$name} if(exists $self->{$name});

	if($name eq 'nntp') {
		# Defer creation until needed.
		printf("AUTOLOAD: creating nntp ...\n") if(not defined $self->{nntp});
		$self->{nntp} = new smNewsFacade if(not defined $self->{nntp});
		return $self->{nntp};
	}

	if($name eq 'newsrc') {
		# Defer creation until needed.
		if(not defined $self->{newsrc}) {
			printf("AUTOLOAD: creating newsrc ...\n");
			$self->{newsrc} = new News::Newsrc;
			$self->{newsrc}->load;
		}

		return $self->{newsrc};
	}

	return $self->{opts}->options if($name eq 'options');

	if(exists $self->{opts}->{$name}) {
		if(@_) {
			$self->{opts}->{$name} = shift;
			$self->{opts}->write;
			return $self->{opts}->{$name};
		} else {
			return $self->{opts}->{$name}
		}
	}

	croak "Can't access `$name' field in class $type";
	return undef;
}
#*---------------------------------------------------------------------------*
sub new {
	my $class = shift;
	my $self = bless {
		opts	=>	undef
	,	mw		=>	undef
	}, $class;

	$self;
}
#*---------------------------------------------------------------------------*
sub parse_args {
	my $self = shift;
	$self->{opts} = new smOpts;

	1;
}
#*---------------------------------------------------------------------------*
sub path_fix {
	my $self = shift;
	my $path = undef;

	$path = File::Spec->join(@_);
	$path =~ s#\\#/#g;

	$path;
}
#*---------------------------------------------------------------------------*
sub path_make {
	my $self = shift;

	mkpath(shift, 0, 0700);
}
#*---------------------------------------------------------------------------*
sub run {
	my $self = shift;

	# Create the main form.
	$self->{mw} = new smfmMain;

	# Put it up and enter MainLoop.
	$self->{mw}->run();

	1;
}
#*---------------------------------------------------------------------------*
sub status_msg {
	my $self = shift;

	$self->{mw}->status_msg(@_);
}
#*---------------------------------------------------------------------------*
sub status_msg_prefix {
	my $self = shift;

	$self->{mw}->status_msg_prefix(@_);
}
#*---------------------------------------------------------------------------*
sub update {
	my $self = shift;

	$self->{mw}->update;
}
#*---------------------------------------------------------------------------*
1;
#*---------------------------------------------------------------------------*
