#* vim:ts=4:
#*---------------------------------------------------------------------------*
#* Project
#*        slurpmedia - Slurp multimedia files from news articles
#*        Copyright (C) 1999  klmcw@earthlink.net <Kevin L. McWhirter>
#*
#*        This program is free software; you can redistribute it and/or modify
#*        it under the terms of the GNU General Public License as published by
#*        the Free Software Foundation; either version 2 of the License, or
#*        (at your option) any later version.
#*
#*        This program is distributed in the hope that it will be useful,
#*        but WITHOUT ANY WARRANTY; without even the implied warranty of
#*        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*        GNU General Public License for more details.
#*
#*        You should have received a copy of the GNU General Public License
#*        along with this program; if not, write to the Free Software
#*        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#*
#* Module
#*        $Id$
#*
#* Description
#*        Subclass News::NNTPClient adding progress indication using main
#*        form's status bar.  Also, fetch_articles utilizes an interface
#*        to make calls to main form callbacks to update the display and
#*        get the actual subject line. mark_articles also uses this interface.
#*---------------------------------------------------------------------------*
#* $Log$
#*---------------------------------------------------------------------------*
package smNewsFacade;

use Carp;
use News::NNTPClient;
@ISA = qw(News::NNTPClient);

use strict qw(subs vars);

#*---------------------------------------------------------------------------*
sub _log_hdr {
	my $self = shift;
	my $grp = shift;
	my $ele = shift;
	my $msg = shift;
	my $dir = $::theApp->path_fix(
		$::theApp->sm_inst_dir
	,	"logs"
	);
	mkdir($dir, 700) unless -d $dir;

	my $file = $::theApp->path_fix(
		$dir
	,	$grp . ".log"
	);
	my $id = msg_id($ele);
	open(FD, ">>$file") or croak "Could not open session log";

	printf(FD "%s%s"	# $ele still has \n
	,	$::theApp->newsrc->marked($grp, $id) ? '*' : ' '
	,	$ele);
	printf(FD "%s", $msg) if(defined $msg);
	printf(FD "\n");

	close(FD);
}
#*---------------------------------------------------------------------------*
# Fetch text from server until single dot.	*NNTPClient Interface*
sub fetch {
    my $self = shift;
    local $/ = "\012";         # Only use LF to account for possible missing CR
    local $\ = "";             # Guarantee that no other EOL is in use
    local $_;

    return unless $self->okprint;

	$self->{fetch_cancelled} = 0;

    my @lines;
    my $line = 0;

    my $SOCK = $self->{SOCK};

    # Loop reading lines until we receive a line with a single period.
    while (<$SOCK>) {
		s/\015?\012$/$self->{EOL}/; # Change termination

		last if($_ eq ".$self->{EOL}");
		return undef if($self->{fetch_cancelled});

		s/^\.\././;		# Fix up escaped dots.

		# Print progress indication
		$line++;
		$::theApp->status_msg("Fetching line %d", $line) if(0 == ($line % 100));
		$::theApp->update;

		push @lines, $_;	# Save each line.
    }

    1 < $self->{DBUG} and	warn "$SOCK received ${\scalar @lines} lines\n";

    wantarray ? @lines : \@lines;
}
#*---------------------------------------------------------------------------*
sub fetch_articles {
	my $self = shift;
	my $obj = shift;
	my @lns = @_;

	my $msg_hdr = 'Fetch Articles > ';
	my $nntp_grp = $::theApp->nntp_grp;

#	$self->fetch_articles_set_msg;

	$self->{oper_cancelled} = 0;

	foreach my $ln (@lns) {
		last if($self->{oper_cancelled});

		#* Utilize Interface with main form.
		my $hdr = $obj->fetch_article_get_header($ln);

		my $article = msg_id($hdr);
		if(not $::theApp->newsrc->marked($nntp_grp, $article)) {
			my $subject = msg_subj($hdr);

#			printf("%s:%s:\n", $article, $subject);

			# fetch will use this to prepend to status_msg.
			$::theApp->status_msg_prefix($msg_hdr . $subject);

			my $dir = $::theApp->path_fix(
				$::theApp->sm_inst_dir
			,	$nntp_grp
			);
			mkdir($dir, 700) unless -d $dir;

			my $deleteit = 0;
			$subject =~ s#[:\\/!\*\"<>?|]#_#g;
			my $file = $::theApp->path_fix(
				$dir
			,	$subject
			);
			my @lns = $self->article($article);
			
			if(not $self->{oper_cancelled}) {
				if(@lns) {
					open(FD, ">$file") or
						die "Could not open file for article $article:$subject";

					map { print FD $_; } @lns or $deleteit = 1;
					close(FD);

					if($deleteit) {
						$::theApp->status_msg("ERROR!");
						unlink($subject);
						last;
					} else {
						$self->_log_hdr($nntp_grp, $hdr);

						$::theApp->newsrc->mark($nntp_grp, $article);
						$::theApp->newsrc->save;
						$::theApp->status_msg("Done.");
					}

					#* Utilize Interface with main form.
					$obj->fetch_article_deselect($ln);
				} elsif($self->{CODE} == 423) {
					$self->_log_hdr($nntp_grp, $hdr . ": " . $self->{MESG});

					$::theApp->newsrc->mark($nntp_grp, $article);
					$::theApp->newsrc->save;
					$::theApp->status_msg($self->{MESG});

					#* Utilize Interface with main form.
					$obj->fetch_article_deselect($ln);
				} else {
					$self->_log_hdr($nntp_grp, $hdr, " Could not fetch!");
					$::theApp->status_msg("Could not Fetch: %s", $self->{CODE});
					$self->{oper_cancelled} = 1;
					return;
				}
			} else {
				#* Utilize Interface with main form.
				$obj->fetch_article_deselect($ln);
			}
		} else {
			#* Utilize Interface with main form.
			$obj->fetch_article_deselect($ln);
		}
	}

	1;
}
#*---------------------------------------------------------------------------*
sub fetch_article_lines {
	my $self = shift;
	my $hdr = shift;
	my $msg_hdr = 'Fetch Article > ';
	my @res = undef;

#	$self->fetch_articles_set_msg;

	$self->{oper_cancelled} = 0;
	return if(not $self->fetch_connect);

	if(not $self->{oper_cancelled}) {
		my $article = msg_id($hdr);
		my $subject = msg_subj($hdr);

		# fetch will use this to prepend to status_msg.
		$::theApp->status_msg_prefix($msg_hdr . $subject);

		@res = $self->article($article);
		if(not $self->{oper_cancelled}) {
			$::theApp->status_msg("Done.");
		}
	}

	$self->quit;

	return wantarray ? @res : \@res;
}
#*---------------------------------------------------------------------------*
sub fetch_connect {
	my $self = shift;

	# Make connection to news server
	$::theApp->status_msg("Connecting to %s", $::theApp->nntp_srvr);
	my $nntp = $self->connect;
	croak "Could not connect to news server." if(not $nntp);
	$self->response;
	if($::theApp->nntp_auth) {
		$self->authinfo($::theApp->nntp_user, $::theApp->nntp_pass) or
			croak "Could not authenticate";
	}

	# Determine range of articles available on news server
	$::theApp->status_msg("Looking at %s", $::theApp->nntp_grp);
	$self->group($::theApp->nntp_grp);
}
#*---------------------------------------------------------------------------*
sub fetch_hdrs {
	my $self = shift;
	my $obj = shift;
	my $msg_hdr = 'Fetch Headers > ';
	my $newsrc = $::theApp->newsrc;
	my $nntp_grp = $::theApp->nntp_grp;

	# fetch will use this to prepend to status_msg.
	$::theApp->status_msg_prefix($msg_hdr);

	my($first, $last) = ($self->fetch_connect);

	return if(not $first);

	# fetch will use this to prepend to status_msg.
	$::theApp->status_msg_prefix($msg_hdr . $nntp_grp);

	$::theApp->status_msg("Expiring Articles...");
	$newsrc->unmark_range($nntp_grp, 0, $first-1);
	$newsrc->save;

	$::theApp->status_msg("Retrieving headers for articles $first - $last");
	$self->{oper_cancelled} = 0;

	my @xhdrs = $self->xpat("", $first, $last);
	$self->quit();

	return if($self->{oper_cancelled});

	$::theApp->status_msg("Removing read articles...");
	my @hdrs = grep {
		chomp;
		$::theApp->update;
		not $newsrc->marked($nntp_grp, msg_id($_));
	} @xhdrs;
	@xhdrs = undef;

	my @rc = ($first, @hdrs);

	@rc;
}
#*---------------------------------------------------------------------------*
sub mark_articles {
	my $self = shift;
	my $obj = shift;

	foreach my $ln (@_) {
		#* Utilize Interface with main form.
		my $hdr = $obj->fetch_article_get_header($ln);

		$::theApp->newsrc->mark($::theApp->nntp_grp, $hdr->{numb});
	}

	$::theApp->newsrc->save;
}
#*---------------------------------------------------------------------------*
sub msg_id {
	my $id = shift;

	$id =~ s/^(\d+)\s+.*/$1/;

	$id;
}
#*---------------------------------------------------------------------------*
sub msg_subj {
	my $subj = shift;

#	$subj =~ s/^\d+\s+(.*)/$1/;

#	$subj;

	my $idx = index($subj, ' ');
	my $rc = substr($subj, $idx + 1, length($subj) - $idx + 1);

	$rc;
}
#*---------------------------------------------------------------------------*
sub new {
	my $class = shift;
	my $self = new News::NNTPClient($::theApp->nntp_srvr);

	if($self) {
		$self->quit;
		$self = bless $self, $class;
		$self->{fetch_cancelled} = 0;
		$self->{oper_cancelled} = 0;
	}

	$self;
}
#*---------------------------------------------------------------------------*
1;
#*---------------------------------------------------------------------------*
