#* vim:ts=4:
#*---------------------------------------------------------------------------*
#* Project
#*        slurpmedia - Slurp multimedia files from news articles
#*        Copyright (C) 1999  klmcw@earthlink.net <Kevin L. McWhirter>
#*
#*        This program is free software; you can redistribute it and/or modify
#*        it under the terms of the GNU General Public License as published by
#*        the Free Software Foundation; either version 2 of the License, or
#*        (at your option) any later version.
#*
#*        This program is distributed in the hope that it will be useful,
#*        but WITHOUT ANY WARRANTY; without even the implied warranty of
#*        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*        GNU General Public License for more details.
#*
#*        You should have received a copy of the GNU General Public License
#*        along with this program; if not, write to the Free Software
#*        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#*
#* Module
#*        $Id$
#*
#* Description
#*
#*---------------------------------------------------------------------------*
#* $Log$
#*---------------------------------------------------------------------------*
package smfmMain;
use strict;
use vars qw/$AUTOLOAD/;

use Carp;
use Tk;
use Tk::widgets qw/Balloon BrowseEntry Button Dialog Listbox ROText/;

require smfmAbout;
require smfmOptions;
require smfmArticle;

#*---------------------------------------------------------------------------*

$smfmMain::colId = 1;
$smfmMain::colSubject = 2;

#*---------------------------------------------------------------------------*
sub _disable_widget {
	my $widget = shift;

	$widget->configure(-state => 'disabled');
}
#*---------------------------------------------------------------------------*
sub _enable_widget {
	my $widget = shift;

	$widget->configure(-state => 'normal');
}
#*---------------------------------------------------------------------------*
sub cb_about {
	my $self = shift;

	$self->{smfmAbout}->Show;
}
#*---------------------------------------------------------------------------*
sub cb_cancel {
	my $self = shift;
	my $nntp = $::theApp->nntp;

	$self->status_msg('Cancelled!');
	$nntp->{fetch_cancelled} = 1;
	$nntp->{oper_cancelled} = 1;
}
#*---------------------------------------------------------------------------*
sub cb_del_selected {
	my $self = shift;

	# busy cursor
	$self->{list}->Busy;
	_enable_widget($self->{btCncl});
	_disable_widget($self->{btDel});
	_disable_widget($self->{btFtch});
	_disable_widget($self->{btRtv});
	_disable_widget($self->{btView});

	# Process selection
	my @sel = $self->{list}->curselection;
	$::theApp->nntp->mark_articles($self, @sel);

	while(@sel = $self->{list}->curselection) {
		$self->{list}->delete(shift @sel);
	}
	$self->_list_write;

	_disable_widget($self->{btCncl});
	_enable_widget($self->{btDel});
	_enable_widget($self->{btFtch});
	_enable_widget($self->{btRtv});
	_enable_widget($self->{btView});

	# restore normal cursor
	$self->{list}->Unbusy;
}
#*---------------------------------------------------------------------------*
sub cb_fetch_hdrs {
	my $self = shift;

	# busy cursor
	$self->{list}->Busy;
	_enable_widget($self->{btCncl});
	_disable_widget($self->{btDel});
	_disable_widget($self->{btFtch});
	_disable_widget($self->{btRtv});
	_disable_widget($self->{btView});

	my($first, @xhdrs) = $::theApp->nntp->fetch_hdrs($self);

	if(@xhdrs) {
		$self->status_msg("Removing duplicates and expired...");
		my %hdrs;
		for ($self->{list}->get(0, 'end')) {
			my $id = smNewsFacade::msg_id($_);

			# Eliminate expired articles.
			next if($id < $first);

			$hdrs{$id} = $_;
		}

		for (@xhdrs) {
			my $id = smNewsFacade::msg_id($_);

			next if(defined $hdrs{$id});

			$hdrs{$id} = $_;
		}

		$self->status_msg("Sorting by subject...");
		# Optimize sort by preparing the sort data in arrays first
		# and use indexes to access those prepared arrays.
		my @subjs = ();
		my @vals = values %hdrs;
		undef %hdrs;

		for (@vals) {
			push @subjs, uc(smNewsFacade::msg_subj($_));
		}
		my @hdrs = @vals [
			sort {
				$::theApp->update if(($smfmMain::a % 100) == 0);
				$subjs[$smfmMain::a] cmp $subjs[$smfmMain::b]
			} 0..$#vals
		];
		undef @subjs;
		undef @vals;

		# Empty list contents
		$self->{list}->delete(0, 'end');

		$self->{list}->insert('end', @hdrs);
		$self->status_msg("%d articles", scalar(@hdrs));
		@hdrs = undef;

		$self->_list_write;
	} else {
		$self->status_msg("no article headers to fetch");
	}

	$self->status_msg_prefix("");

	_disable_widget($self->{btCncl});
	_enable_widget($self->{btDel});
	_enable_widget($self->{btFtch});
	_enable_widget($self->{btRtv});
	_enable_widget($self->{btView});

	# restore normal cursor
	$self->{list}->Unbusy;
}
#*---------------------------------------------------------------------------*
sub cb_go_sel {
	my $self = shift;

	my @sel = $self->{list}->curselection;
	$self->{list}->yview(shift @sel) if(@sel);
	@sel = undef;

	1;
}
#*---------------------------------------------------------------------------*
sub cb_grp_change {
	my $self = shift;
	my $widget = shift;
	my $sel = shift;

	# First save current group state
	$self->_list_write;

	$::theApp->nntp_grp($sel);
	$self->_set_title;
	$self->_list_read;

	1;
}
#*---------------------------------------------------------------------------*
sub cb_options {
	my $self = shift;

	$self->{smfmOptions}->Show;
}
#*---------------------------------------------------------------------------*
sub cb_quit {
	my $self = shift;

	# Perform any cleanup before app shutdown.
	$self->{mw}->destroy;
}
#*---------------------------------------------------------------------------*
#* Implement interface with smNewsFacade
sub fetch_article_deselect {
	my $self = shift;
	my $ln = shift;

	$self->{list}->selectionClear($ln);
}
#*---------------------------------------------------------------------------*
#* Implement interface with smNewsFacade
sub fetch_article_get_header {
	my $self = shift;
	my $ln = shift;

	$self->{list}->get($ln); 
}
#*---------------------------------------------------------------------------*
sub cb_retrieve_selected {
	my $self = shift;

	# busy cursor
#	$self->{list}->Busy;
	_enable_widget($self->{btCncl});
	_disable_widget($self->{btDel});
	_disable_widget($self->{btFtch});
	_disable_widget($self->{btRtv});
	_disable_widget($self->{btView});

	# Process selection
	# If more articles are selected during retrieve,
	# process them also. Note that if articles not retrieved
	# yet are de-selected, they will not be processed.
	my(@sel, @csel);

	# Allow fetch_articles to be called one article at a time.
	# Will only connect if needs to.
	my $nntp = $::theApp->nntp;

	$self->status_msg_prefix('Fetch Articles > ');

	if($nntp->fetch_connect and not $nntp->{oper_cancelled}) {
		while(@csel = $self->{list}->curselection) {
			my $csel = shift @csel;
			push @sel, $csel;

			$nntp->fetch_articles($self, $csel);

			last if($nntp->{oper_cancelled});
		}
		@csel = undef;

		if(not $nntp->{oper_cancelled}) {
			# Reselect them
			$self->{list}->selectionClear(0, 'end');

			# Sort them so they are deleted in ascending order.
			# They may have been 'push'ed out of sequence
			# if selecting took place during retrieve operation.
			@csel = sort { $smfmMain::a <=> $smfmMain::b; } @sel;
			@sel = undef;

			map { $self->{list}->selectionSet($_); } @csel;
			@csel = undef;

			# deletes by selection have to be handled carefully!
			while(@sel = $self->{list}->curselection) {
				my $sel = shift @sel;
				$self->{list}->delete($sel);
			}
			@sel = undef;
			$self->_list_write;
		}

		$nntp->quit;
	}

	_disable_widget($self->{btCncl});
	_enable_widget($self->{btDel});
	_enable_widget($self->{btFtch});
	_enable_widget($self->{btRtv});
	_enable_widget($self->{btView});

	# restore normal cursor
#	$self->{list}->Unbusy;
}
#*---------------------------------------------------------------------------*
sub cb_view_selected {
	my $self = shift;

	# busy cursor
	$self->{list}->Busy;
	_enable_widget($self->{btCncl});
	_disable_widget($self->{btDel});
	_disable_widget($self->{btFtch});
	_disable_widget($self->{btRtv});
	_disable_widget($self->{btView});

	# Process 1st selection only!
	my ($sel, @rest) = $self->{list}->curselection;
	my $ro = new smfmArticle($self->{mw}
	,	$::theApp->nntp
	,	$self->{list}->get($sel)
	);

	# smfmArticle keeps refs to its own children so they stay around
	# until dismissed.
	$ro = undef;

	_disable_widget($self->{btCncl});
	_enable_widget($self->{btDel});
	_enable_widget($self->{btFtch});
	_enable_widget($self->{btRtv});
	_enable_widget($self->{btView});

	# restore normal cursor
	$self->{list}->Unbusy;
}
#*---------------------------------------------------------------------------*
sub _initialize {
my $self = shift;
my $mw = $self->{mw};

$mw->title('Slurp Media');

#* Define Menu bar and menus
my $menubar = $mw->Frame;
$menubar->grid(qw/-sticky ew/);
$menubar->gridColumnconfigure(qw/2 -weight 1/);

my $mnFile = $menubar->Menubutton(qw/-text File -tearoff 0 -underline 0 -menuitems/ =>
[
	[Button    => '~Quit', -command => [$self => 'cb_quit']],
]);
$mnFile->grid(qw/-row 0 -column 0 -sticky w/); 

#my $mnEdit = $menubar->Menubutton(qw/-text Edit -tearoff 0 -underline 0 -menuitems/ =>
#[
#	[Button	=>	'Cu~t']
#,	[Button	=>	'~Copy']
#,	[Button	=>	'~Paste']
#]);
#$mnEdit->grid(qw/-row 0 -column 1 -sticky w/); 

my $mnAbout = $menubar->Menubutton(qw/-text Help -tearoff 0 -underline 0 -menuitems/ =>
[
	[Button    => '~About Slurp Media', -command => [$self => 'cb_about']],
]);
$mnAbout->grid(qw/-row 0 -column 2 -sticky e/); 

#* Define Tool bar and buttons
my $toolbar = $mw->Frame(qw/-borderwidth 2 -relief groove/);
$toolbar->grid(qw/-sticky ew/);
my $col = 0;

$self->{btQuit} =
	$toolbar->Button(qw/-text Q -command/ => [$self => 'cb_quit']);
$self->{btQuit}->grid(qw/-row 0 -column/, $col++, qw/-sticky w/);

$self->{btOpts} =
	$toolbar->Button(qw/-text O -command/ => [$self => 'cb_options']);
$self->{btOpts}->grid(qw/-row 0 -column/, $col++, qw/-sticky w/);

$self->{btGoSel} =
	$toolbar->Button(qw/-text G -command/ => [$self => 'cb_go_sel']);
$self->{btGoSel}->grid(qw/-row 0 -column/, $col++, qw/-sticky w/);

$self->{btCncl} =
	$toolbar->Button(qw/-text C -command/ => [$self => 'cb_cancel']);
$self->{btCncl}->grid(qw/-row 0 -column/, $col++, qw/-sticky w/);

$self->{btView} =
	$toolbar->Button(qw/-text V -command/ => [$self => 'cb_view_selected']);
$self->{btView}->grid(qw/-row 0 -column/, $col++, qw/-sticky w/);

$self->{btDel} =
	$toolbar->Button(qw/-text D -command/ => [$self => 'cb_del_selected']);
$self->{btDel}->grid(qw/-row 0 -column/, $col++, qw/-sticky w/);

$self->{btFtch} =
	$toolbar->Button(qw/-text F -command/ => [$self => 'cb_fetch_hdrs']);
$self->{btFtch}->grid(qw/-row 0 -column/, $col++, qw/-sticky w/);

$self->{btRtv} =
	$toolbar->Button(qw/-text R -command/ => [$self => 'cb_retrieve_selected']);
$self->{btRtv}->grid(qw/-row 0 -column/, $col++, qw/-sticky w/);

# :TODO: FIX ME! Generates not CODE reference at line 130 of BrowseEntry.pm
@smfmMain::grps = $::theApp->newsrc->groups;
$self->{beGrps} = $toolbar->BrowseEntry(
	-browsecmd	=> [$self => 'cb_grp_change']
,	-variable	=> \$::theApp->nntp_grp
,	-width		=>	40
);
$self->{beGrps}->configure(-choices => \@smfmMain::grps);
$self->{beGrps}->grid(qw/-row 0 -column/, $col++, qw/-sticky e/);

$toolbar->gridColumnconfigure($col, qw/-weight 1/);

#* Add text area for column headings
#* :TODO: We'll use tags to provide column bindings
my $tarea = $mw->ROText(
	-background	=> 'black'
,	-foreground	=> 'white'
,	-font		=> 'courier 10'
,	-height     => 1 
,	-setgrid    => 1
);
$tarea->grid(qw/-sticky ew/);
$tarea->insert('end', "ID     Subject");

#* Add Listbox for list of articles
my $list = $mw->Scrolled('Listbox',
	-scrollbars	=> 'osoe'
,	-font		=> 'courier 10'
,	-height		=> 30
,	-width		=> 120
,	-selectmode	=> 'extended'
);
$list->grid(qw/-sticky nsew/);
$self->{list} = $list;

#* Add Label to use for the status bar
my $sbar = $mw->Label(-textvariable => \$self->{svar}, qw/-anchor w/);
$sbar->grid(qw/-sticky ew/);

#* Create Balloon help
my $balloon = $mw->Balloon(-statusbar => $sbar);
my $msg;
$msg = "Cancel Retrieve";
$balloon->attach($self->{btCncl}
,	"-balloonmsg"	=>	$msg
,	"-statusmsg"	=>	$msg
);
$msg = "Delete Selection";
$balloon->attach($self->{btDel}
,	"-balloonmsg"	=>	$msg
,	"-statusmsg"	=>	$msg
);
$msg = "Fetch Article Subjects";
$balloon->attach($self->{btFtch}
,	"-balloonmsg"	=>	$msg
,	"-statusmsg"	=>	$msg
);
$msg = "Goto First Selected Article";
$balloon->attach($self->{btGoSel}
,	"-balloonmsg"	=>	$msg
,	"-statusmsg"	=>	$msg
);
$msg = "Edit Slurp Media Options";
$balloon->attach($self->{btOpts}
,	"-balloonmsg"	=>	$msg
,	"-statusmsg"	=>	$msg
);
$msg = "Quit Slurp Media";
$balloon->attach($self->{btQuit}
,	"-balloonmsg"	=>	$msg
,	"-statusmsg"	=>	$msg
);
$msg = "Retrieve Selected Articles";
$balloon->attach($self->{btRtv}
,	"-balloonmsg"	=>	$msg
,	"-statusmsg"	=>	$msg
);
$msg = "View Selected Article";
$balloon->attach($self->{btView}
,	"-balloonmsg"	=>	$msg
,	"-statusmsg"	=>	$msg
);

# This is enabled when an operation is in progress.
_disable_widget($self->{btCncl});

# These will be enabled after a fetch.
_disable_widget($self->{btDel});
_disable_widget($self->{btRtv});
_disable_widget($self->{btView});

#* Return undef on error, otherwise return $self
	$self;
}
#*---------------------------------------------------------------------------*
sub _list_file_path {
	my $self = shift;
	my $dir = $::theApp->path_fix(
		$::theApp->sm_inst_dir
	,	$::theApp->nntp_grp
	);
	$::theApp->path_make($dir);

	$::theApp->path_fix(
		$dir
	,	"list_state.txt"
	);
}
#*---------------------------------------------------------------------------*
sub _list_read {
	my $self = shift;
	my $file = $self->_list_file_path;

	# Empty list contents
	$self->{list}->delete(0, 'end');

	open(FD, "<$file") or return;
	my @hdrs = map { s/\r//g;s/\n//g;$_; } <FD>;
	close(FD);

	if(@hdrs) {
		$self->{list}->insert('end', @hdrs);
		$self->status_msg('%d articles', scalar(@hdrs));
		_enable_widget($self->{btDel});
		_enable_widget($self->{btRtv});
		_enable_widget($self->{btView});
	}
}
#*---------------------------------------------------------------------------*
sub _list_write {
	my $self = shift;
	my $file = $self->_list_file_path;

	open(FD, ">$file") or return;
	map { printf(FD "%s\n", $_); } $self->{list}->get(0, 'end');
	close(FD);
}
#*---------------------------------------------------------------------------*
sub _set_title {
	my $self = shift;

	$self->{mw}->title($::theApp->nntp_grp . ' - Slurp Media');
}
#*---------------------------------------------------------------------------*
sub new {
	my $class = shift;

	my $self = bless {
		mw			=>	Tk::MainWindow->new
	,	beGrps		=>	undef
	,	btCncl		=>	undef
	,	btDel		=>	undef
	,	btFtch		=>	undef
	,	btGoSel		=>	undef
	,	btOpts		=>	undef
	,	btQuit		=>	undef
	,	btRtv		=>	undef
	,	btView		=>	undef
	,	list		=>	undef
	,	svar		=>	""
	,	curr_prefix	=>	undef

	,	smfmAbout	=>	undef
	,	smfmOptions	=>	undef

	,	xhList		=>	undef
	,	xhIter		=>	undef
	}, $class;

	$self->{smfmAbout} = new smfmAbout($self->{mw});
	$self->{smfmOptions} = new smfmOptions($self->{mw});

	$self->_initialize;
	$self->_list_read;
	$self->_set_title;

	$self;
}
#*---------------------------------------------------------------------------*
sub run {
	MainLoop;
}
#*---------------------------------------------------------------------------*
sub status_msg {
	my $self = shift;
	my $msg = shift;

	# Allow varargs
	$msg = sprintf($msg, @_) if(@_);

	# Prepend prefix if defined
	$msg = $self->{curr_prefix} . " : " . $msg if($self->{curr_prefix});

	$self->{svar} = $msg;

	# Make sure update takes place.
	$self->update;
}
#*---------------------------------------------------------------------------*
sub status_msg_prefix {
	my $self = shift;
	my $msg = shift;

	$msg = sprintf($msg, @_) if(@_);

	# If prefix is being cleared, undef it so it will not be prepended
	# in status_msg.
	$self->{curr_prefix} = ($msg ne "") ? $msg : undef;
}
#*---------------------------------------------------------------------------*
sub update {
	my $self = shift;

	$self->{mw}->update;
}
#*---------------------------------------------------------------------------*
1;
#*---------------------------------------------------------------------------*
